@@ -402,15 +402,18 @@ func GetAPIServerAltNames(cfg *kubeadmapi.InitConfiguration) (*certutil.AltNames
 		return nil, errors.Wrapf(err, "unable to get first IP address from the given CIDR: %v", cfg.Networking.ServiceSubnet)
 	}
 
+	var dnsNames []string
+	if len(cfg.NodeRegistration.Name) > 0 {
+		dnsNames = append(dnsNames, cfg.NodeRegistration.Name)
+	}
+	dnsNames = append(dnsNames, "kubernetes", "kubernetes.default", "kubernetes.default.svc")
+	if len(cfg.Networking.DNSDomain) > 0 {
+		dnsNames = append(dnsNames, fmt.Sprintf("kubernetes.default.svc.%s", cfg.Networking.DNSDomain))
+	}
+
 	// create AltNames with defaults DNSNames/IPs
 	altNames := &certutil.AltNames{
-		DNSNames: []string{
-			cfg.NodeRegistration.Name,
-			"kubernetes",
-			"kubernetes.default",
-			"kubernetes.default.svc",
-			fmt.Sprintf("kubernetes.default.svc.%s", cfg.Networking.DNSDomain),
-		},
+		DNSNames: dnsNames,
 		IPs: []net.IP{
 			internalAPIServerVirtualIP,
 			advertiseAddress,
@@ -458,9 +461,16 @@ func getAltNames(cfg *kubeadmapi.InitConfiguration, certName string) (*certutil.
 			cfg.LocalAPIEndpoint.AdvertiseAddress)
 	}
 
+	var dnsNames []string
+	if len(cfg.NodeRegistration.Name) > 0 {
+		dnsNames = []string{cfg.NodeRegistration.Name, "localhost"}
+	} else {
+		dnsNames = []string{"localhost"}
+	}
+
 	// create AltNames with defaults DNSNames/IPs
 	altNames := &certutil.AltNames{
-		DNSNames: []string{cfg.NodeRegistration.Name, "localhost"},
+		DNSNames: dnsNames,
 		IPs:      []net.IP{advertiseAddress, net.IPv4(127, 0, 0, 1), net.IPv6loopback},
 	}
 
@@ -707,13 +717,15 @@ func NewSelfSignedCACert(cfg *CertConfig, key crypto.Signer) (*x509.Certificate,
 			CommonName:   cfg.CommonName,
 			Organization: cfg.Organization,
 		},
-		DNSNames:              []string{cfg.CommonName},
 		NotBefore:             notBefore,
 		NotAfter:              notAfter,
 		KeyUsage:              keyUsage,
 		BasicConstraintsValid: true,
 		IsCA:                  true,
 	}
+	if len(cfg.CommonName) > 0 {
+		tmpl.DNSNames = []string{cfg.CommonName}
+	}
 
 	certDERBytes, err := x509.CreateCertificate(cryptorand.Reader, &tmpl, &tmpl, key.Public(), key)
 	if err != nil {
