@@ -754,7 +754,7 @@ func applySchedulingGatedCondition(pod *api.Pod) {
 		}
 	}
 
-	pod.Status.Conditions = append(pod.Status.Conditions, api.PodCondition{
+	podutil.UpdatePodCondition(&pod.Status, &api.PodCondition{
 		Type:    api.PodScheduled,
 		Status:  api.ConditionFalse,
 		Reason:  apiv1.PodReasonSchedulingGated,
@@ -765,10 +765,6 @@ func applySchedulingGatedCondition(pod *api.Pod) {
 // applyAppArmorVersionSkew implements the version skew behavior described in:
 // https://github.com/kubernetes/enhancements/tree/master/keps/sig-node/24-apparmor#version-skew-strategy
 func applyAppArmorVersionSkew(ctx context.Context, pod *api.Pod) {
-	if !utilfeature.DefaultFeatureGate.Enabled(features.AppArmorFields) {
-		return
-	}
-
 	if pod.Spec.OS != nil && pod.Spec.OS.Name == api.Windows {
 		return
 	}
