@@ -109,7 +109,7 @@ type CloudNodeController struct {
 
 	nodesLister corelisters.NodeLister
 	nodesSynced cache.InformerSynced
-	workqueue   workqueue.RateLimitingInterface
+	workqueue   workqueue.TypedRateLimitingInterface[string]
 }
 
 // NewCloudNodeController creates a CloudNodeController object
@@ -120,9 +120,6 @@ func NewCloudNodeController(
 	nodeStatusUpdateFrequency time.Duration,
 	workerCount int32) (*CloudNodeController, error) {
 
-	eventBroadcaster := record.NewBroadcaster()
-	recorder := eventBroadcaster.NewRecorder(scheme.Scheme, v1.EventSource{Component: "cloud-node-controller"})
-
 	_, instancesSupported := cloud.Instances()
 	_, instancesV2Supported := cloud.InstancesV2()
 	if !instancesSupported && !instancesV2Supported {
@@ -132,14 +129,15 @@ func NewCloudNodeController(
 	cnc := &CloudNodeController{
 		nodeInformer:              nodeInformer,
 		kubeClient:                kubeClient,
-		broadcaster:               eventBroadcaster,
-		recorder:                  recorder,
 		cloud:                     cloud,
 		nodeStatusUpdateFrequency: nodeStatusUpdateFrequency,
 		workerCount:               workerCount,
 		nodesLister:               nodeInformer.Lister(),
 		nodesSynced:               nodeInformer.Informer().HasSynced,
-		workqueue:                 workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "Nodes"),
+		workqueue: workqueue.NewTypedRateLimitingQueueWithConfig(
+			workqueue.DefaultTypedControllerRateLimiter[string](),
+			workqueue.TypedRateLimitingQueueConfig[string]{Name: "Nodes"},
+		),
 	}
 
 	// Use shared informer to listen to add/update of nodes. Note that any nodes
@@ -156,7 +154,21 @@ func NewCloudNodeController(
 // This controller updates newly registered nodes with information
 // from the cloud provider. This call is blocking so should be called
 // via a goroutine
+//
+//logcheck:context // RunWithContext should be used instead of Run in code which supports contextual logging.
 func (cnc *CloudNodeController) Run(stopCh <-chan struct{}, controllerManagerMetrics *controllersmetrics.ControllerManagerMetrics) {
+	cnc.RunWithContext(wait.ContextForChannel(stopCh), controllerManagerMetrics)
+}
+
+// RunWithContext will sync informer caches and starting workers.
+// This controller updates newly registered nodes with information
+// from the cloud provider. This call is blocking so should be called
+// via a goroutine
+func (cnc *CloudNodeController) RunWithContext(ctx context.Context, controllerManagerMetrics *controllersmetrics.ControllerManagerMetrics) {
+	cnc.broadcaster = record.NewBroadcaster(record.WithContext(ctx))
+	cnc.recorder = cnc.broadcaster.NewRecorder(scheme.Scheme, v1.EventSource{Component: "cloud-node-controller"})
+	stopCh := ctx.Done()
+
 	defer utilruntime.HandleCrash()
 	defer cnc.workqueue.ShutDown()
 
@@ -178,16 +190,16 @@ func (cnc *CloudNodeController) Run(stopCh <-chan struct{}, controllerManagerMet
 
 	// The periodic loop for updateNodeStatus polls the Cloud Provider periodically
 	// to reconcile the nodes addresses and labels.
-	go wait.Until(func() {
-		if err := cnc.UpdateNodeStatus(context.TODO()); err != nil {
+	go wait.UntilWithContext(ctx, func(ctx context.Context) {
+		if err := cnc.UpdateNodeStatus(ctx); err != nil {
 			klog.Errorf("failed to update node status: %v", err)
 		}
-	}, cnc.nodeStatusUpdateFrequency, stopCh)
+	}, cnc.nodeStatusUpdateFrequency)
 
 	// These workers initialize the nodes added to the cluster,
 	// those that are Tainted with TaintExternalCloudProvider.
 	for i := int32(0); i < cnc.workerCount; i++ {
-		go wait.Until(cnc.runWorker, time.Second, stopCh)
+		go wait.UntilWithContext(ctx, cnc.runWorker, time.Second)
 	}
 
 	<-stopCh
@@ -196,34 +208,26 @@ func (cnc *CloudNodeController) Run(stopCh <-chan struct{}, controllerManagerMet
 // runWorker is a long-running function that will continually call the
 // processNextWorkItem function in order to read and process a message on the
 // workqueue.
-func (cnc *CloudNodeController) runWorker() {
-	for cnc.processNextWorkItem() {
+func (cnc *CloudNodeController) runWorker(ctx context.Context) {
+	for cnc.processNextWorkItem(ctx) {
 	}
 }
 
 // processNextWorkItem will read a single work item off the workqueue and
 // attempt to process it, by calling the syncHandler.
-func (cnc *CloudNodeController) processNextWorkItem() bool {
+func (cnc *CloudNodeController) processNextWorkItem(ctx context.Context) bool {
 	obj, shutdown := cnc.workqueue.Get()
 	if shutdown {
 		return false
 	}
 
 	// We wrap this block in a func so we can defer cnc.workqueue.Done.
-	err := func(obj interface{}) error {
-		defer cnc.workqueue.Done(obj)
-
-		var key string
-		var ok bool
-		if key, ok = obj.(string); !ok {
-			cnc.workqueue.Forget(obj)
-			utilruntime.HandleError(fmt.Errorf("expected string in workqueue but got %#v", obj))
-			return nil
-		}
+	err := func(key string) error {
+		defer cnc.workqueue.Done(key)
 
 		// Run the syncHandler, passing it the key of the
 		// Node resource to be synced.
-		if err := cnc.syncHandler(key); err != nil {
+		if err := cnc.syncHandler(ctx, key); err != nil {
 			// Put the item back on the workqueue to handle any transient errors.
 			cnc.workqueue.AddRateLimited(key)
 			klog.Infof("error syncing '%s': %v, requeuing", key, err)
@@ -232,7 +236,7 @@ func (cnc *CloudNodeController) processNextWorkItem() bool {
 
 		// Finally, if no error occurs we Forget this item so it does not
 		// get queued again until another change happens.
-		cnc.workqueue.Forget(obj)
+		cnc.workqueue.Forget(key)
 		return nil
 	}(obj)
 
@@ -245,14 +249,14 @@ func (cnc *CloudNodeController) processNextWorkItem() bool {
 }
 
 // syncHandler implements the logic of the controller.
-func (cnc *CloudNodeController) syncHandler(key string) error {
+func (cnc *CloudNodeController) syncHandler(ctx context.Context, key string) error {
 	_, name, err := cache.SplitMetaNamespaceKey(key)
 	if err != nil {
 		utilruntime.HandleError(fmt.Errorf("invalid resource key: %s", key))
 		return nil
 	}
 
-	return cnc.syncNode(context.TODO(), name)
+	return cnc.syncNode(ctx, name)
 }
 
 // UpdateNodeStatus updates the node status, such as node addresses
@@ -446,31 +450,9 @@ func (cnc *CloudNodeController) syncNode(ctx context.Context, nodeName string) e
 	})
 
 	err = clientretry.RetryOnConflict(UpdateNodeSpecBackoff, func() error {
-		var curNode *v1.Node
-		if cnc.cloud.ProviderName() == "gce" {
-			// TODO(wlan0): Move this logic to the route controller using the node taint instead of condition
-			// Since there are node taints, do we still need this?
-			// This condition marks the node as unusable until routes are initialized in the cloud provider
-			if err := nodeutil.SetNodeCondition(cnc.kubeClient, types.NodeName(nodeName), v1.NodeCondition{
-				Type:               v1.NodeNetworkUnavailable,
-				Status:             v1.ConditionTrue,
-				Reason:             "NoRouteCreated",
-				Message:            "Node created without a route",
-				LastTransitionTime: metav1.Now(),
-			}); err != nil {
-				return err
-			}
-
-			// fetch latest node from API server since GCE-specific condition was set and informer cache may be stale
-			curNode, err = cnc.kubeClient.CoreV1().Nodes().Get(context.TODO(), nodeName, metav1.GetOptions{})
-			if err != nil {
-				return err
-			}
-		} else {
-			curNode, err = cnc.nodeInformer.Lister().Get(nodeName)
-			if err != nil {
-				return err
-			}
+		curNode, err = cnc.nodeInformer.Lister().Get(nodeName)
+		if err != nil {
+			return err
 		}
 
 		newNode := curNode.DeepCopy()
@@ -478,7 +460,7 @@ func (cnc *CloudNodeController) syncNode(ctx context.Context, nodeName string) e
 			modify(newNode)
 		}
 
-		_, err = cnc.kubeClient.CoreV1().Nodes().Update(context.TODO(), newNode, metav1.UpdateOptions{})
+		_, err = cnc.kubeClient.CoreV1().Nodes().Update(ctx, newNode, metav1.UpdateOptions{})
 		if err != nil {
 			return err
 		}
