@@ -89,9 +89,9 @@ run_create_job_tests() {
     create_and_use_new_namespace
 
     # Test kubectl create job
-    kubectl create job test-job --image=k8s.gcr.io/nginx:test-cmd
+    kubectl create job test-job --image=registry.k8s.io/nginx:test-cmd
     # Post-Condition: job nginx is created
-    kube::test::get_object_assert 'job test-job' "{{${image_field0:?}}}" 'k8s.gcr.io/nginx:test-cmd'
+    kube::test::get_object_assert 'job test-job' "{{${image_field0:?}}}" 'registry.k8s.io/nginx:test-cmd'
     # Clean up
     kubectl delete job test-job "${kube_flags[@]}"
 
@@ -141,3 +141,89 @@ run_kubectl_create_kustomization_directory_tests() {
   set +o nounset
   set +o errexit
 }
+
+has_one_of_error_message() {
+  local message=$1
+  local match1=$2
+  local match2=$3
+
+  if (grep -q "${match1}" <<< "${message}") || (grep -q "${match2}" <<< "${message}"); then
+    echo "Successful"
+    echo "message:${message}"
+    echo "has either:${match1}"
+    echo "or:${match2}"
+    return 0
+  else
+    echo "FAIL!"
+    echo "message:${message}"
+    echo "has neither:${match1}"
+    echo "nor:${match2}"
+    caller
+    return 1
+  fi
+}
+
+# Runs tests related to kubectl create --validate
+run_kubectl_create_validate_tests() {
+  set -o nounset
+  set -o errexit
+
+  create_and_use_new_namespace
+
+   ## test --validate no value expects default strict is used
+   kube::log::status "Testing kubectl create --validate"
+   # create and verify
+   output_message=$(! kubectl create -f hack/testdata/invalid-deployment-unknown-and-duplicate-fields.yaml --validate 2>&1)
+   has_one_of_error_message "${output_message}" 'strict decoding error' 'error validating data'
+
+  ## test --validate=true
+  kube::log::status "Testing kubectl create --validate=true"
+  # create and verify
+  output_message=$(! kubectl create -f hack/testdata/invalid-deployment-unknown-and-duplicate-fields.yaml --validate=true 2>&1)
+  has_one_of_error_message "${output_message}" 'strict decoding error' 'error validating data'
+
+  ## test  --validate=false
+  kube::log::status "Testing kubectl create --validate=false"
+  # create and verify
+  output_message=$(kubectl create -f hack/testdata/invalid-deployment-unknown-and-duplicate-fields.yaml --validate=false)
+  kube::test::if_has_string "${output_message}" "deployment.apps/invalid-nginx-deployment created"
+  # cleanup
+  kubectl delete deployment invalid-nginx-deployment
+
+  ## test --validate=strict
+  kube::log::status "Testing kubectl create --validate=strict"
+  # create and verify
+  output_message=$(! kubectl create -f hack/testdata/invalid-deployment-unknown-and-duplicate-fields.yaml --validate=strict 2>&1)
+  has_one_of_error_message "${output_message}" 'strict decoding error' 'error validating data'
+
+  ## test --validate=warn
+  kube::log::status "Testing kubectl create --validate=warn"
+  # create and verify
+  output_message=$(kubectl create -f hack/testdata/invalid-deployment-unknown-and-duplicate-fields.yaml --validate=warn)
+  kube::test::if_has_string "${output_message}" "deployment.apps/invalid-nginx-deployment created"
+  # cleanup
+  kubectl delete deployment invalid-nginx-deployment
+
+  ## test  --validate=ignore
+  kube::log::status "Testing kubectl create --validate=ignore"
+  # create and verify
+  output_message=$(kubectl create -f hack/testdata/invalid-deployment-unknown-and-duplicate-fields.yaml --validate=ignore)
+  kube::test::if_has_string "${output_message}" "deployment.apps/invalid-nginx-deployment created"
+  # cleanup
+  kubectl delete deployment invalid-nginx-deployment
+
+  ## test default is strict validation
+  kube::log::status "Testing kubectl create"
+  # create and verify
+  output_message=$(! kubectl create -f hack/testdata/invalid-deployment-unknown-and-duplicate-fields.yaml 2>&1)
+  has_one_of_error_message "${output_message}" 'strict decoding error' 'error validating data'
+
+  ## test invalid validate value
+  kube::log::status "Testing kubectl create --validate=foo"
+  # create and verify
+  output_message=$(! kubectl create -f hack/testdata/invalid-deployment-unknown-and-duplicate-fields.yaml --validate=foo 2>&1)
+  kube::test::if_has_string "${output_message}" 'invalid - validate option "foo"'
+
+  set +o nounset
+  set +o errexit
+}
